# Generated by Django 2.0.4 on 2018-04-07 01:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailusers', '0006_userprofile_prefered_language'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='current_time_zone',
            field=models.CharField(default='', help_text='Select your current time zone', max_length=40, verbose_name='current time zone'),
        ),
    ]
