ul.tagit{border:1px solid #c6c6c6;background:inherit}ul.tagit li.tagit-choice{border-radius:6px;-webkit-border-radius:6px;border:1px solid #cad8f3;background:none;background-color:#dee7f8;font-weight:400}ul.tagit li.tagit-choice .tagit-label:not(a){color:#555}ul.tagit li.tagit-choice a.tagit-close{text-decoration:none}ul.tagit li.tagit-choice .tagit-close{right:.4em}ul.tagit li.tagit-choice .ui-icon{display:none}ul.tagit li.tagit-choice .tagit-close .text-icon{display:inline;font-family:arial,sans-serif;font-size:16px;line-height:16px;color:#777}ul.tagit li.tagit-choice.remove,ul.tagit li.tagit-choice:hover{background-color:#bbcef1;border-color:#6d95e0}ul.tagit li.tagit-choice a.tagit-close .text-icon:hover,ul.tagit li.tagit-choice a.tagLabel:hover{color:#222}ul.tagit input[type=text]{color:#333;background:none}.ui-widget{font-size:1.1em}