{% extends "wagtailadmin/block_forms/sequence.html" %}
{% load i18n %}

{% block footer %}
    <button type="button" title="Add" id="{{ prefix }}-add" class="c-sf-add-button c-sf-add-button--visible"><i aria-hidden="true">+</i></button>
{% endblock %}
