{% load i18n %}
{% load wagtailadmin_tags %}

{% comment %}
    Expects a variable 'page', a page object.
    Tests whether that page is locked for editing, and if so, outputs a 'locked' icon.
{% endcomment %}

{% if page.locked %}
    {# TODO: Different look when it's locked by current user? #}
    <span class="indicator locked-indicator icon icon-locked" title="{% trans 'This page is locked to further editing' %}"></span>
{% endif %}
