<fieldset>
    <legend>{{ self.heading }}</legend>
    <ul class="fields">
        {% for child in self.children %}
            <li class="{{ child.classes|join:" " }}">{{ child.render_as_field }}</li>
        {% endfor %}
    </ul>
</fieldset>
