{% extends "tests/base.html" %}

{% block content %}
    <p>Thank you for submitting a Support Request.</p>
    <ul>
      {% for key, value in form_submission.get_data.items %}
        <li>{{ key }}: {{ value }}</li>
      {% endfor %}
    </ul>
{% endblock %}
