// Generated by CoffeeScript 1.6.2
(function() {
    (function(jQuery) {
        return jQuery.widget('IKS.hallohr', {
            options: {
                editable: null,
                toolbar: null,
                uuid: '',
                buttonCssClass: null
            },
            populateToolbar: function(toolbar) {
                var buttonElement, buttonset;

                buttonset = jQuery('<span class="' + this.widgetName + '"></span>');
                buttonElement = jQuery('<span></span>');
                buttonElement.hallobutton({
                    uuid: this.options.uuid,
                    editable: this.options.editable,
                    label: 'Horizontal rule',
                    command: 'insertHorizontalRule',
                    icon: 'icon-horizontalrule',
                    cssClass: this.options.buttonCssClass
                });
                buttonset.append(buttonElement);
                buttonset.hallobuttonset();
                return toolbar.append(buttonset);
            }
        });
    })(jQuery);

}).call(this);
