# Generated by Django 2.2.dev20181026000358 on 2018-10-27 09:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtaildocs', '0008_document_file_size'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='document',
            options={'verbose_name': 'document', 'verbose_name_plural': 'documents'},
        ),
    ]
