.replace-file-input{display:inline-block;position:relative;overflow:hidden;padding-bottom:2px}.replace-file-input [type=file]{padding:0;opacity:0;position:absolute;top:0;right:0;direction:ltr;width:auto;display:block;font-size:5em}.replace-file-input:hover,.replace-file-input [type=file]:hover{cursor:pointer}.replace-file-input:hover button{background-color:#00676a}.upload-list>li{padding:1em}.upload-list .left{text-align:center;word-break:break-all}.upload-list .preview{width:150px;min-height:150px;display:block;position:relative;text-align:center;max-width:100%;margin:auto}.upload-list .progress,.upload-list .thumb,.upload-list .thumb:before,.upload-list canvas,.upload-list img{position:absolute;max-width:100%}.upload-list .progress{box-shadow:0 0 5px 2px hsla(0,0%,100%,.4);z-index:4;top:60%;left:20%;right:20%;width:60%}.upload-list .thumb{top:0;right:0;bottom:0;left:0;z-index:1;width:100%}.upload-list .thumb:before,.upload-list canvas,.upload-list img{left:0;right:0;top:0;bottom:0;margin:auto}.upload-list .thumb:before{z-index:2;top:0;width:1em;font-size:10em;line-height:1.4em;color:#f0f0f0}.upload-list canvas,.upload-list img{z-index:3}.upload-list .hasthumb:before,.upload-list .status-msg{display:none}.upload-list .upload-complete .progress{opacity:0}.upload-list .upload-success .status-msg.success{display:block}.upload-list .upload-failure{border-color:#cd3238}.upload-list .upload-failure .preview{display:none}.upload-list .upload-failure .status-msg.failure{display:block}.upload-list .upload-server-error{border-color:#cd3238}.upload-list .upload-server-error .preview{display:none}.upload-list .upload-server-error .status-msg.server-error{display:block}