# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-01-24 15:04
from django.db import migrations, models
import modelcluster.fields


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0013_auto_20161220_1957'),
    ]

    operations = [
        migrations.CreateModel(
            name='EventCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
            ],
        ),
        migrations.AddField(
            model_name='eventpage',
            name='categories',
            field=modelcluster.fields.ParentalManyToManyField(blank=True, to='tests.EventCategory'),
        ),
    ]
