{% load i18n %}

<form action="{{ edit_action }}" method="POST" enctype="multipart/form-data" novalidate>
    <ul class="fields">
        {% csrf_token %}
        {% for field in form %}
            {% if field.is_hidden %}
                {{ field }}
            {% else %}
                {% include "wagtailadmin/shared/field_as_li.html" %}
            {% endif %}
        {% endfor %}
        <li>
            <input type="submit" value="{% trans 'Update' %}" class="button" />
            <a href="{{ delete_action }}" class="delete button button-secondary no">{% trans "Delete" %}</a>
        </li>
    </ul>
</form>
