<li class="menu-item{% if active %} menu-active{% endif %}">
    <a href="#" data-nav-primary-submenu-trigger class="submenu-trigger {{ classnames }}"{{ attr_string }}>{{ label }}</a>
    <div class="nav-submenu">
        <h2 id="nav-submenu-{{ name }}-title" class="{{ classnames }}">{{ label }}</h2>
        <ul class="nav-submenu__list" aria-labelledby="nav-submenu-{{ name }}-title">
            {{ menu_html }}
        </ul>
        {% block menu_footer %}{% endblock %}
    </div>
</li>
