{% load i18n %}
<li data-inline-panel-child id="inline_child_{{ child.form.prefix }}">
    {% if child.form.non_field_errors %}
    <ul>
        {% for error in child.form.non_field_errors %}
            <li class="error-message">
                <span>{{ error|escape }}</span>
            </li>
        {% endfor %}
    </ul>
    {% endif %}
    <ul class="controls">
        {% if can_order %}
            <li><button type="button" class="button icon text-replace white icon-order-up inline-child-move-up" id="{{ child.form.prefix }}-move-up" title="{% trans 'Move up' %}">{% trans "Move up" %}</button></li>
            <li><button type="button" class="button icon text-replace white icon-order-down inline-child-move-down" id="{{ child.form.prefix }}-move-down" title="{% trans 'Move down' %}">{% trans "Move down" %}</button></li>
        {% endif %}
        <li><button type="button" class="button icon text-replace white icon-bin hover-no" id="{{ child.form.DELETE.id_for_label }}-button" title="{% trans 'Delete' %}">{% trans "Delete" %}</button></li>
    </ul>
    {{ child.render_form_content }}
</li>
