# Generated by Django 2.2.7 on 2019-11-29 10:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0041_group_collection_permissions_verbose_name_plural'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pagerevision',
            name='approved_go_live_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='approved go live at'),
        ),
    ]
