.content header,.result-list{margin-bottom:0}.listing td,.listing th{vertical-align:top}.listing thead th.sorted a{color:#007d7e}.listing tbody{overflow:auto}.listing tbody tr:hover ul.actions{visibility:visible}.listing tbody tr>td{background-color:inherit}.listing tbody tr>td a.edit-obj{color:inherit;font-weight:600}.changelist-filter{padding:0 15px}.changelist-filter h2{background-color:#fafafa;font-size:13px;line-height:31px;margin-top:0;padding-left:8px;border-bottom:1px solid #e6e6e6}.changelist-filter h3{font-size:12px;margin-bottom:0}.changelist-filter ul{padding-left:0;margin-bottom:25px}.changelist-filter li{list-style-type:none;margin:0 0 4px;padding-left:0}.changelist-filter a{font-family:Open Sans,Arial,sans-serif;border-radius:3px;width:auto;line-height:1.2em;padding:8px 12px;font-size:.9em;font-weight:400;vertical-align:middle;display:block;background-color:#fff;border:1px solid #43b1b0;color:#43b1b0;text-decoration:none;text-transform:uppercase;position:relative;overflow:hidden;box-sizing:border-box;-webkit-font-smoothing:auto;-moz-appearance:none}.changelist-filter a:hover{background-color:#358c8b;border-color:#358c8b;color:#fff}.changelist-filter li.selected a{color:#fff!important;border-color:#43b1b0!important;background-color:#43b1b0}.no-search-results{margin-top:30px}.no-search-results h2{padding-top:.3em;margin-bottom:.3em}.no-search-results img{float:left;margin:0 15px 15px 0;width:50px}.pagination{margin-top:3em;border-top:1px dashed #d9d9d9;padding:2em 1em 0}.pagination ul{margin-top:-1.25em}p.no-results{margin:30px 1em 0}@media screen and (min-width:50em){.changelist-filter{float:right;padding:0 1.5%}.result-list{padding:0 1.5% 0 0}.result-list.col12{padding-right:0}.result-list.col12 tbody td:last-child{padding-right:50px}.pagination,.result-list tbody th:first-child{padding-left:50px}.pagination{padding-right:50px}.pagination.col9{width:73.5%}p.no-results{margin:30px 50px 0}}@media screen and (min-width:75em){.result-list.col9{width:79%}.changelist-filter{width:21%}.pagination.col9{width:77.5%}}