# Generated by Django 2.1.2 on 2018-10-27 10:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0036_streamfieldmigrationmodels_add_field'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='customdocument',
            options={'verbose_name': 'document', 'verbose_name_plural': 'documents'},
        ),
    ]
