{% load i18n %}
<nav class="listing-filter" aria-labelledby="other-searches-title">
    <h3 id="other-searches-title" class="filter-title">{% trans "Other searches" %}</h3>
    <ul class="filter-options">
        {{ options_html }}
    </ul>
</nav>
