.listing .field label{border:0;clip:rect(0 0 0 0);height:1px;margin:-1px;overflow:hidden;padding:0;position:absolute;width:1px}.listing .field label:active,.listing .field label:focus{clip:auto;height:auto;margin:0;overflow:visible;position:static;width:auto}.listing input,.listing select,.listing textarea{font-size:1em}.listing select+span:after{font-size:2.5em!important}.page-permissions-listing .admin_page_chooser .field-content{width:100%}